@@ -48,6 +48,12 @@ typedef void        (*sc_sig_t) (int);
 #include <Windows.h>
 #endif
 
+#if _POSIX_C_SOURCE >= 199309L
+#include <time.h>
+#else
+#include <unistd.h>
+#endif
+
 typedef struct sc_package
 {
   int                 is_registered;
@@ -124,6 +130,42 @@ static int          sc_num_packages = 0;
 static int          sc_num_packages_alloc = 0;
 static sc_package_t *sc_packages = NULL;
 
+void
+sc_extern_c_hack_1 (void)
+{
+  /* Completing the hack in sc.h on providing the prototype.
+     We use the macro SC_EXTER_C_BEGIN; after including all headers
+     and before declaring functions to ensure C linkage. */
+}
+
+void
+sc_extern_c_hack_2 (void)
+{
+  /* Completing the hack in sc.h on providing the prototype.
+     We use the macro SC_EXTER_C_END; after declaring all functions,
+     just before the final include-once check, to ensure C linkage. */
+}
+
+int
+sc_mpi_is_enabled (void)
+{
+#ifdef SC_ENABLE_MPI
+  return 1;
+#else
+  return 0;
+#endif
+}
+
+int
+sc_mpi_is_shared (void)
+{
+#ifdef SC_ENABLE_MPISHARED
+  return 1;
+#else
+  return 0;
+#endif
+}
+
 #ifdef SC_ENABLE_PTHREAD
 
 static pthread_mutex_t sc_default_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -1271,7 +1313,6 @@ sc_init (sc_MPI_Comm mpicomm,
          int catch_signals, int print_backtrace,
          sc_log_handler_t log_handler, int log_threshold)
 {
-  int                 w;
   const char         *trace_file_name;
   const char         *trace_file_prio;
 
@@ -1337,27 +1378,10 @@ sc_init (sc_MPI_Comm mpicomm,
     }
   }
 
-  w = 24;
+  /* one line of logging if the threshold is not SC_LP_SILENT */
   SC_GLOBAL_ESSENTIALF ("This is %s\n", SC_PACKAGE_STRING);
-#if 0
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "F77", SC_F77);
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "FFLAGS", SC_FFLAGS);
-#endif
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "CPP", SC_CPP);
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "CPPFLAGS", SC_CPPFLAGS);
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "CC", SC_CC);
-#if 0
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "C_VERSION", SC_C_VERSION);
-#endif
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "CFLAGS", SC_CFLAGS);
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "LDFLAGS", SC_LDFLAGS);
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "LIBS", SC_LIBS);
-#if 0
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "BLAS_LIBS", SC_BLAS_LIBS);
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "LAPACK_LIBS", SC_LAPACK_LIBS);
-  SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "FLIBS", SC_FLIBS);
-#endif
-
+  SC_GLOBAL_INFOF ("MPI is enabled %d shared %d\n",
+                   sc_mpi_is_enabled (), sc_mpi_is_shared ());
   sc_initialized = 1;
 }
 
@@ -1591,6 +1615,16 @@ sc_version_point (void)
 }
 #endif
 
+int
+sc_is_littleendian (void)
+{
+  /* We use the volatile keyword to deactivate compiler optimizations related
+   * to the variable uint.
+   */
+  const volatile uint32_t uint = 1;
+  return *(char *) &uint == 1;
+}
+
 int
 sc_have_zlib (void)
 {
@@ -1610,3 +1644,25 @@ sc_have_json (void)
   return 1;
 #endif
 }
+
+void
+sc_sleep (unsigned milliseconds){
+#if _POSIX_C_SOURCE >= 199309L
+  struct timespec ts;
+  /* full seconds */
+  ts.tv_sec = milliseconds / 1000;
+  /* nanoseconds */
+  ts.tv_nsec = (milliseconds % 1000) * 1000000;
+  nanosleep (&ts, NULL);
+#elif defined(_POSIX_C_SOURCE)
+  /* older POSIX */
+  if (milliseconds >= 1000) {
+    sleep (milliseconds / 1000);
+  }
+  usleep ((milliseconds % 1000) * 1000);
+#elif _MSC_VER
+  Sleep (milliseconds);
+#else
+  SC_ABORT ("No suitable sleep function available.");
+#endif
+}
